"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const js_yaml_1 = __importDefault(require("js-yaml"));
const lodash_pick_1 = __importDefault(require("lodash.pick"));
const encoder_1 = require("@mesg/api/lib/util/encoder");
const replaceVariable = (value, env) => {
    if (typeof value !== 'string')
        return value;
    const reg = new RegExp('\\$\\(env\\:(.*)\\)', 'g');
    return value.replace(reg, (_, p1) => {
        if (!Object.keys(env).includes(p1)) {
            throw new Error('env variable ' + p1 + ' must be set');
        }
        return env[p1];
    });
};
const decode = (content, env) => {
    const data = js_yaml_1.default.safeLoad(content.toString());
    return JSON.parse(JSON.stringify(data), function (_, value) {
        return replaceVariable(value, env);
    });
};
const mapToArray = (inputs) => Object.keys(inputs || {}).map(key => (Object.assign(Object.assign({}, inputs[key]), { key })));
const parseParams = (params) => mapToArray(params).map(x => (Object.assign(Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description', 'type', 'repeated', 'optional'])), { object: parseParams(x.object) })));
exports.service = (content) => __awaiter(void 0, void 0, void 0, function* () {
    const definition = decode(content, {});
    return Object.assign(Object.assign({}, lodash_pick_1.default(definition, ['sid', 'name', 'description', 'repository'])), { configuration: definition.configuration || {}, dependencies: mapToArray(definition.dependencies), tasks: mapToArray(definition.tasks).map(x => (Object.assign(Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description'])), { inputs: parseParams(x.inputs), outputs: parseParams(x.outputs) }))), events: mapToArray(definition.events).map(x => (Object.assign(Object.assign({}, lodash_pick_1.default(x, ['key', 'name', 'description'])), { data: parseParams(x.data) }))) });
});
const nodeCompiler = (type, def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
    const nodes = {
        result: (def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return ({
                key,
                taskKey: def.taskKey,
                instanceHash: opts.instanceResolver ? yield opts.instanceResolver(def) : def.instanceHash
            });
        }),
        event: (def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return ({
                key,
                eventKey: def.eventKey,
                instanceHash: opts.instanceResolver ? yield opts.instanceResolver(def) : def.instanceHash
            });
        }),
        task: (def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return ({
                key,
                taskKey: def.taskKey,
                instanceHash: opts.instanceResolver ? yield opts.instanceResolver(def) : def.instanceHash
            });
        }),
        map: (def, key, opts) => __awaiter(void 0, void 0, void 0, function* () {
            return ({
                key,
                outputs: Object.keys(def).map(key => (Object.assign({ key }, (typeof def[key] === 'object' && def[key].key // if the value is an object containing an attribute key
                    ? {
                        ref: {
                            key: def[key].key,
                            nodeKey: def[key].stepKey || opts.defaultNodeKey,
                        }
                    }
                    : {
                        constant: encoder_1.encodeField(def[key])
                    }))))
            });
        }),
        filter: (def, key) => __awaiter(void 0, void 0, void 0, function* () {
            return ({
                key,
                conditions: Object.keys(def.conditions).map(key => ({
                    key,
                    predicate: 1,
                    value: def.conditions[key]
                }))
            });
        })
    };
    return {
        [type]: yield nodes[type](def, key, opts)
    };
});
exports.process = (content, instanceResolver, envs) => __awaiter(void 0, void 0, void 0, function* () {
    const definition = decode(content, envs);
    let nodes = [];
    let edges = [];
    let previousKey = null;
    let previousKeyWithOutputs = null;
    let i = 0;
    for (const step of definition.steps) {
        step.key = step.key || `node-${i}`;
        if (step.inputs) {
            const mapKey = `${step.key}-inputs`;
            const mapNode = yield nodeCompiler('map', step.inputs, mapKey, { defaultNodeKey: previousKeyWithOutputs });
            nodes.push(mapNode);
            if (previousKey) {
                edges.push({ src: previousKey, dst: mapKey });
            }
            previousKeyWithOutputs = previousKey = mapKey;
            i++;
        }
        const type = step.type !== 'trigger'
            ? step.type
            : step.eventKey
                ? 'event'
                : 'result';
        const stepNode = yield nodeCompiler(type, step, step.key, { instanceResolver });
        nodes.push(stepNode);
        if (previousKey) {
            edges.push({ src: previousKey, dst: step.key });
        }
        if (type !== 'filter') {
            previousKeyWithOutputs = step.key;
        }
        previousKey = step.key;
        i++;
    }
    return {
        key: definition.key,
        nodes,
        edges,
    };
});
